@@ -4,7 +4,7 @@ import sys
 import types
 import traceback
 
-from deferred import Deferred, defer
+from callback import Callback, defer
 
 from twisted_utils import mergeFunctionMetadata
 
@@ -16,13 +16,13 @@ except ImportError:
     class TwistedDeferred: pass
 
 
-def launch(df):
-    def eb(e):
-        if isinstance(e, Exception):
-            import traceback
-            import sys
-            traceback.print_exception(type(e), e, sys.exc_info()[2])
-    df.add_callback(eb)
+class Return(object):
+    def __init__(self, value=None):
+        self.value = value
+
+
+class InvalidYieldException(Exception):
+    pass
 
 
 def format_tb(e):
@@ -35,9 +35,9 @@ def format_tb(e):
     return first + s + "\n" + last
 
 
-def _monocle_chain(to_gen, g, deferred):
+def _monocle_chain(to_gen, g, callback):
     # This function is complicated by the need to prevent unbounded recursion
-    # arising from repeatedly yielding immediately ready deferreds.  This while
+    # arising from repeatedly yielding immediately ready callbacks.  This while
     # loop and the state variable solve that by manually unfolding the
     # recursion.
 
@@ -52,44 +52,47 @@ def _monocle_chain(to_gen, g, deferred):
                 from_gen = g.send(to_gen)
         except StopIteration:
             # "return" statement (or fell off the end of the generator)
-            from_gen = None
+            from_gen = Return()
         except Exception, e:
             tb = traceback.format_exc()
             if not hasattr(e, "_monocle"):
                 e._monocle = {'tracebacks': []}
             e._monocle['tracebacks'].append(tb)
-            deferred.callback(e)
-            return deferred
+            callback(e)
+            return callback
 
-        if not isinstance(from_gen,
-                          (Deferred, TwistedDeferred)):
-            deferred.callback(from_gen)
-            return deferred
+        if isinstance(from_gen, Return):
+            callback(from_gen.value)
+            return callback
+        elif not isinstance(from_gen,
+                            (Callback, TwistedDeferred)):
+            e = InvalidYieldException("Unexpected value '%s' of type '%s' yielded from o-routine '%s'.  O-routines can only yield Callback and Return types." % (from_gen, type(from_gen), g))
+            return _monocle_chain(e, g, callback)
 
         state = {'waiting': True}
 
-        # a deferred was yielded, get the result.
+        # a callback was yielded, get the result.
         def gotResult(r):
             if state['waiting']:
                 state['waiting'] = False
                 state['result'] = r
             else:
-                _monocle_chain(r, g, deferred)
+                _monocle_chain(r, g, callback)
         if isinstance(from_gen, TwistedDeferred):
             from_gen.addBoth(gotResult)
         else:
-            from_gen.add_callback(gotResult)
+            from_gen.add(gotResult)
 
         if state['waiting']:
             # Haven't called back yet, set flag so that we get reinvoked
             # and return from the loop
             state['waiting'] = False
-            return deferred
+            return callback
 
         to_gen = state['result']
 
 
-def maybeDeferredGenerator(f, *args, **kw):
+def maybeCallbackGenerator(f, *args, **kw):
     try:
         result = f(*args, **kw)
     except Exception, e:
@@ -100,8 +103,8 @@ def maybeDeferredGenerator(f, *args, **kw):
         return defer(e)
 
     if isinstance(result, types.GeneratorType):
-        return _monocle_chain(None, result, Deferred())
-    elif isinstance(result, Deferred):
+        return _monocle_chain(None, result, Callback())
+    elif isinstance(result, Callback):
         return result
     elif isinstance(result, TwistedDeferred):
         return result  # FIXME -- convert
@@ -111,41 +114,54 @@ def maybeDeferredGenerator(f, *args, **kw):
 # @_o
 def _o(f):
     """
-    monocle helps you write Deferred-using code that looks like a regular
+    monocle helps you write Callback-using code that looks like a regular
     sequential function.  For example::
 
         @_o
         def foo():
-            result = yield makeSomeRequestResultingInDeferred()
+            result = yield makeSomeRequestResultingInCallback()
             print result
 
-    When you call anything that results in a Deferred, you can simply yield it;
-    your generator will automatically be resumed when the Deferred's result is
-    available. The generator will be sent the result of the Deferred with the
+    When you call anything that results in a Callback, you can simply yield it;
+    your generator will automatically be resumed when the Callback's result is
+    available. The generator will be sent the result of the Callback with the
     'send' method on generators, or if the result was a failure, 'throw'.
 
-    Your coroutine-enabled generator will return a Deferred object, which
-    will result in the return value of the generator (or will fail with a
-    failure object if your generator raises an unhandled exception). Note that
-    you can't use "return result" to return a value; use "yield result"
-    instead. Falling off the end of the generator, or simply using "return"
-    will cause the Deferred to have a result of None.
+    Your coroutine-enabled generator will return a Callback object,
+    which will result in the return value of the generator (or will
+    fail with a failure object if your generator raises an unhandled
+    exception). Note that you can't use "return result" to return a
+    value; use "yield Return(result)" instead. Falling off the end of
+    the generator, or simply using "return" will cause the Callback to
+    have a result of None.  Yielding anything other and a Callback or
+    a Return is not allowed, and will raise an exception.
 
-    The Deferred returned from your generator will call back with an
+    The Callback returned from your generator will call back with an
     exception if your generator raised an exception::
 
         @_o
         def foo():
-            result = yield makeSomeRequestResultingInDeferred()
+            result = yield makeSomeRequestResultingInCallback()
             if result == 'foo':
-                # this will become the result of the Deferred
-                yield 'success'
+                # this will become the result of the Callback
+                yield Return('success')
             else:
                 # this too
                 raise Exception('fail')
     """
     def unwindGenerator(*args, **kwargs):
-        return maybeDeferredGenerator(f, *args, **kwargs)
+        return maybeCallbackGenerator(f, *args, **kwargs)
     return mergeFunctionMetadata(f, unwindGenerator)
-
 o = _o
+
+@_o
+def launch(oroutine, *args, **kwargs):
+    r = yield oroutine(*args, **kwargs)
+    if isinstance(r, Exception):
+        if hasattr(r, '_monocle'):
+            print format_tb(r)
+        else:
+            import traceback
+            import sys
+            traceback.print_exception(type(r), r, sys.exc_info()[2])
+    yield Return(r)
